#ifndef WVISCSOLVER_H
#define WVISCSOLVER_H

#include "dart.h"
#include "wBLRegion.h"
namespace dart
{

class DART_API ViscSolver
{
public:
    double Cdt,
           Cdf,
           Cdp;
    
    bool printOn = true;

    std::vector<std::vector<BLRegion *>> Sections;

private:
    double Re,
           CFL0;
    
    int solverExitCode;

    std::vector<bool> stagPtMvmt;

    TimeSolver *tSolver;

public:
    ViscSolver(double _Re, double _Minf, double _CFL0, unsigned int nSections);
    ~ViscSolver();
    int Run(unsigned int couplIter);
    double GetRe() const {return Re;};

    void SetGlobMesh(size_t iSec, size_t iRegion, std::vector<double> _x, std::vector<double> _y, std::vector<double> _z);
    void SetVelocities(size_t iSec, size_t iRegion, std::vector<double> _vx, std::vector<double> _vy, std::vector<double> _vz);
    void SetMe(size_t iSec, size_t iRegion, std::vector<double> _Me);
    void SetRhoe(size_t iSec, size_t iRegion, std::vector<double> _Rhoe);
    void SetxxExt(size_t iSec, size_t iRegion, std::vector<double> _xxExt);
    void SetDeltaStarExt(size_t iSec, size_t iRegion, std::vector<double> _DeltaStarExt);

    std::vector<double> ExtractBlowingVel(size_t iSec, size_t iRegion) const;
    std::vector<double> Extractxx(size_t iSec, size_t iRegion) const;
    std::vector<double> ExtractDeltaStar(size_t iSec, size_t iRegion) const;
    std::vector<double> ExtractxCoord(size_t iSec, size_t iRegion) const;

    double Getxtr(size_t iSec, size_t iRegion) const {return Sections[iSec][iRegion]->xtr;};

private:
    void CheckWaves(size_t iPoint, BLRegion *bl);
    void AverageTransition(size_t iPoint, BLRegion *bl, int forced);
    void ComputeDrag(std::vector<BLRegion *> const &bl);
    void ComputeBlwVel();
    void PrintBanner();
};
} // namespace dart
#endif //WVISCSOLVER_H